"""
Module that contains plotting routines for single systems.

The idea to do this is to provide the user with some quick 
commands to plot the evolution of a system

There is no preloaded matplotlib rc, you should do that yourself
"""

import pandas as pd
import numpy as np

import matplotlib.pyplot as plt
from david_phd_functions.plotting.plot_functions import (
    plot_orbit,
    plot_masses,
    plot_HR_diagram,
)

from binarycpython.utils.functions import get_arg_keys, output_lines
from binarycpython.utils.run_system_wrapper import run_system
from binarycpython.utils.custom_logging_functions import binary_c_log_code

# Define the custom_logging_strings. These are kept to the minimum necessary for each plotting routine.

custom_logging_string_masses = ""
custom_logging_string_orbit = ""

custom_logging_string_HR_diagram = """
Printf("HR_PLOTTING %30.12e %d %d %g %g %g %g %g %g\\n",
    //
    stardata->model.time, // 1
    
    // stellar types
    stardata->star[0].stellar_type, //2
    stardata->star[1].stellar_type, //3

    // luminosity and radii
    stardata->star[0].luminosity, // 4
    stardata->star[1].luminosity, // 5
    stardata->star[0].radius,     // 6
    stardata->star[1].radius,     // 7

    // masses
    stardata->star[0].pms_mass, // 8
    stardata->star[1].pms_mass  // 9
    );
"""


# Define the parse functions for the plotting routines
def dummy():
    pass


def parse_function_hr_diagram(output):
    """
    Parsing function for the HR plotting routine
    """

    # extract info from the single evolution

    values_list = []

    parameters = [
        "time",
        "stellar_type_1",
        "stellar_type_2",
        "luminosity_1",
        "luminosity_2",
        "radius_1",
        "radius_2",
        "pms_mass_1",
        "pms_mass_2",
    ]

    # Go over the output.
    for el in output_lines(output):
        headerline = el.split()[0]

        # Check the header and act accordingly
        if headerline == "HR_PLOTTING":
            values = el.split()[1:]
            values_list.append(values)

    df = pd.DataFrame(values_list)
    df.columns = parameters

    df = df.astype(np.float64)
    df["stellar_type_1"] = df["stellar_type_1"].astype(np.int64)
    df["stellar_type_2"] = df["stellar_type_2"].astype(np.int64)

    return df


def plot_system(plot_type, **kwargs):
    """
    TODO: Complex Function!
    TODO: make sure this way of passing args works correctly.
    TODO: make the plotting specific keywords available via the inspect stuff
    Function to plot the evolution of the system.  

    This goes (in general) via the following steps:
        - a preset custom logging for a specific plotting routine is loaded.
        - This is used for the run_system call
        - The output of this run_system is loaded into a dataframe by parsing it with a corresponding parsing function
        - The dataframe is passed to the plotting routine
        - plot is shown or returned.

    There are several pre-set plots to choose from

    All keywords are considered kwargs, except for plot_type
    input:
        plot_type: string input should be one of the following types: ['mass_evolution', 'orbit_evolution', 'hr_diagram'].
            Input will be matched against this, and then go through a dictionary to pick the correct plotting function. 
        return_fig: boolean whether to return the fig object instead of plotting the plot (makes so that you can customize it)
        show_stellar_types: whether to plot the stellar type evolution on a second pane. This is not included in all the plotting routines.
        Other input: other kwargs that are passed to run_system (inspect the docstring of run_system for more info)
    """

    # set defaults and options
    return_fig = False
    show_stellar_types = False

    plot_types_dict = {
        "mass_evolution": {
            "plot_function": plot_masses,
            "custom_logging_string": custom_logging_string_masses,
            "parse_function": dummy,
        },
        "orbit_evolution": {
            "plot_function": plot_orbit,
            "custom_logging_string": custom_logging_string_orbit,
            "parse_function": dummy,
        },
        "hr_diagram": {
            "plot_function": plot_HR_diagram,
            "custom_logging_string": custom_logging_string_HR_diagram,
            "parse_function": parse_function_hr_diagram,
        },
    }
    plot_system_specific_keywords = ["plot_type", "show_plot", "show_stellar_types"]

    # First check on the plot_type input
    if not plot_type in plot_types_dict.keys():
        print(
            "Warning, the provided plot type is not known. Please choose one from the following:\n\t{}".format(
                plot_types_dict.keys()
            )
        )
        raise ValueError

    # First: check all the arguments. Chosen to not check all the keywords for run_system and binary_c specifically, but just to pick out the ones needed for this routine. run_system will handle the rest
    run_system_arg_dict = {}

    for key in kwargs.keys():
        if key == "show_plot":
            show_plot = kwargs[key]
        elif key == "show_stellar_types":
            show_stellar_types = kwargs[key]

        # The rest will be passed to run_system
        else:
            run_system_arg_dict[key] = kwargs[key]

    # TODO: When a list of plot_types is passed, make it so that the strings are chained, and that the output of the binary_c call is handled by multiple parsers
    custom_logging_code = binary_c_log_code(
        plot_types_dict[plot_type]["custom_logging_string"]
    )
    run_system_arg_dict["custom_logging_code"] = custom_logging_code

    run_system_arg_dict["parse_function"] = plot_types_dict[plot_type]["parse_function"]

    # Run and get the output of the parse function
    binary_c_output_df = run_system(**run_system_arg_dict)

    fig = plot_types_dict[plot_type]["plot_function"](
        binary_c_output_df, show_plot=show_plot, show_stellar_types=show_stellar_types
    )

    if not show_plot:
        return fig


plot_system(
    plot_type="hr_diagram",
    M_1=10,
    M_2=5,
    orbital_period=100000,
    max_evolution_time=15000,
    show_plot=True,
)
